/* mess_avc: UNIX command-/event-driven audio-volume-controller.
 * Module: main program.
 * @(#)mess_avc.c	1.1	25jan2013 MJS
 *
 * Copyright (c) 2013 by Mike Spooner
 *----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 *
 * Core API:    ANSI X3.159-1989 C Language
 * Other APIs:  IEEE Std 1003.1-1988 (POSIX.1)
 *		MJS Portable C Library
 ***********************************************************************
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include "mjsu.h"
#include "audctl.h"

static const char *version = "@(#)mess_avc	1.1	25jan2013 MJS";
static int fd = -1;
static INT origvol = 0;

static VOID handle_interrupt(VOID)
	{
	audioctl_setvol(fd, origvol);
	audioctl_close(fd);
	exit(EXIT_SUCCESS);
	}

static VOID catch_interrupts(VOID)
	{
	static const INT sigs[] =
		{SIGHUP,SIGINT,SIGQUIT,SIGPIPE,SIGTERM,SIGKILL};
	struct sigaction action;
	int i;

	memset(&action, 0, sizeof(action));
	action.sa_handler = handle_interrupt;
	sigemptyset(&action.sa_mask);
	for (i = 0; sigs[i] != SIGKILL; ++i)
		sigaddset(&action.sa_mask, sigs[i]);

	for (i = 0; sigs[i] != SIGKILL; ++i)
		sigaction(sigs[i], &action, NULL);
	}

INT main(INT ac, CHAR **av)
	{
	BOOL show_version = NO;
	SHORT upswitch = 3, dnswitch = 1;
	INT volume;
	CHAR pkt[126+1];
	const CHAR *e = "unknown error";

	getflags(&ac, &av,
		"dn#,up#,version", "",
		&dnswitch, &upswitch, &show_version);

	if (show_version)
		printf("%s\n", version+4);
	if ((dnswitch < 1) || (dnswitch > 4))
		error("-dn# must be between 1 and 4");
	if ((upswitch < 1) || (upswitch > 4))
		error("-up# must be between 1 and 4");
	if (upswitch == dnswitch)
		error("-up# cannot be same as -dn#");

	/* convert switch-numbers into bit-masks */
	upswitch = 1 << (upswitch - 1);
	dnswitch = 1 << (dnswitch - 1);

	/* open audio-control device */
	if ((fd = audioctl_open()) < 0)
		{
		e = strerror(errno);
		error("could not open audio control: %s", e);
		}

	/* arrange to tidy up on interrupt or exit */
	origvol = audioctl_getvol(fd);
	catch_interrupts();

	/* process switch states, until there are no more */
	while (fgets(pkt, sizeof(pkt), stdin))
		{
		volume = audioctl_getvol(fd);

		if (pkt[0] & 0x40)	/* is valid switch state */
			{
			if (pkt[0] & upswitch)
				{
				if (pkt[0] & dnswitch)
					{
					/* both switches: mute */
					audioctl_setvol(fd, 0);
					}
				else
					audioctl_setvol(fd, volume + 6);
				}
			else if (pkt[0] & dnswitch)
				audioctl_setvol(fd, volume - 6);
			}
		else
			{
			audioctl_setvol(fd, 0);
			warning("input error: 0x%X", pkt[0]);
			}
		}
	
	audioctl_close(fd);
	return (EXIT_SUCCESS);
	}
