/* pdlbox: monitor and event-reporter for the MJS PedalBox device.
 * Module: main program.
 * @(#)pdlbox.c	1.3	25jan2013 MJS
 *
 * Copyright (c) 2013 by Mike Spooner
 *----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 *
 * Core API:    ANSI X3.159-1989 C Language
 * Other APIs:  IEEE Std. 1003.1-1988 (POSIX.1)
 *		MJS Portable C Library
 ***********************************************************************
 */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>
#include "mjsu.h"
#include "pdlbox.h"

static const char *version = "@(#)pdlbox	1.3	25jan2013 MJS";
static int fd = -1;

static VOID reset_devs(VOID)
	{
	pdl_close(fd);
	}

static VOID handle_env_signal(VOID)
	{
	reset_devs();
	exit(EXIT_SUCCESS);
	}

static VOID catch_env_signals(VOID)
	{
	static const INT sigs[] =
		{SIGHUP,SIGINT,SIGQUIT,SIGPIPE,SIGTERM,SIGKILL};
	struct sigaction action;
	int i;

	memset(&action, 0, sizeof(action));
	action.sa_handler = handle_env_signal;
	sigemptyset(&action.sa_mask);
	for (i = 0; sigs[i] != SIGKILL; ++i)
		sigaddset(&action.sa_mask, sigs[i]);

	for (i = 0; sigs[i] != SIGKILL; ++i)
		sigaction(sigs[i], &action, NULL);
	}

INT main(INT ac, CHAR **av)
	{
	BOOL show_version = NO, autorepeat = NO, probeonly = NO;
	INT pedals, prev = -1, interval;
	CHAR pkt[3];

	getflags(&ac, &av,
		"p,r,version", "<serial-port>",
		&probeonly, &autorepeat, &show_version);

	if (show_version)
		printf("%s\n", version+4);
	
	if ((ac <= 0) || !*av)
		usage("<serial-port>");

	atexit(reset_devs);
	catch_env_signals();

	if ((fd = pdl_open(*av)) < 0)
		error("\"%s\" - %s", *av, strerror(errno));
	if (probeonly)
		exit (EXIT_SUCCESS);

	pkt[0] = '@';
	pkt[1] = '\r';
	pkt[2] = '\n';
	interval = 1;
	for (pedals = pdl_sense(fd), prev = -1, interval = 1;
			pdl_sleep(fd, interval);
			prev = pedals, pedals = pdl_sense(fd))
		{
		if (autorepeat)
			{
			if (!prev && !pedals)
				continue;
			}
		else if (pedals == prev)
			continue;

		if (pedals & PDL_ERROR)
			error("pedalbox disconnected");
		else
			{
			/* encode pedal state into MESS protocol packet */
			*pkt = 0x40;
			if (pedals & PDL_OTHER)
				*pkt |= 0x02;
			if (pedals & PDL_LEFT)
				*pkt |= 0x01;
			if (pedals & PDL_RIGHT)
				*pkt |= 0x04;

			/* emit MESS packet */
			write(STDOUT_FILENO, pkt, sizeof(pkt));

			/* if L&R pedals are on, wait longer for next sample */
			if ((*pkt & 0x5) == 0x5)
				interval = 2;
			else
				interval = 1;
			}
		}
	error("pdlbox disconnected");

	pdl_close(fd);
	return (EXIT_SUCCESS);
	}
