/* pdlbox: monitor and event-reporter for the MJS PedalBox device.
 * Module: POSIX PedalBox device interface.
 * @(#)serport.c	1.3	22jan2013 MJS
 *
 * Copyright (c) 2013 by Mike Spooner
 *----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *----------------------------------------------------------------------
 *
 * Core API:    ANSI X3.159-1989 C Language
 * Other APIs:  IEEE Std. 1003.1-1988 (POSIX.1)
 *		MJS Portable C Library
 ***********************************************************************
 */
#include <unistd.h>
#if defined(__sun) && defined(__SVR4)
#define __EXTENSIONS__
#endif
#include <termios.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include "mjsu.h"
#include "pdlbox.h"

#ifndef O_NONBLOCK
#ifdef O_NDELAY
#define O_NONBLOCK	O_NDELAY
#else
#error "no non-blocking open() facility"
#endif
#endif


static struct termios saved_tty;
static struct termios tty;

/* sleep for a specified number of deciseconds, armed with an open
 * "termios" file-descriptor.
 */
INT pdl_sleep(INT fd, INT decisec)
	{
	CHAR ch;

	tcgetattr(fd, &tty);
	tty.c_cc[VMIN] = 0;
	tty.c_cc[VTIME] = decisec;  /* 1 = 0.1-second read-timeout */
	tcsetattr(fd, TCSANOW, &tty);
	read(fd, &ch, 1); /* wait until timeout */
	return (1);
	}

static BOOL pdl_ping(INT fd)
	{
	CHAR ch;

	write(fd, "#", 1);
	tcgetattr(fd, &tty);
	tty.c_cc[VMIN] = 0;
	tty.c_cc[VTIME] = 1;
	tcsetattr(fd, TCSADRAIN, &tty);
	if (read(fd, &ch, 1) <= 0)
		return (NO);
	return (YES);
	}


static BOOL pdl_probe(INT fd)
	{
	static const INT bauds[] = {B1200, B2400, B4800, B0};
	static const CHAR *magic = "\023\03\030\153\033";
	static const INT widths[] = {CS7, CS8, CS5};
	INT bits, b, w;
	const CHAR *p;
	CHAR ch;

	/* now probe the pedalbox hardware: it should exactly echo at all
	 * baud-rates and valid framings!
	 */
	tty.c_cc[VMIN] = 0;
	tty.c_cc[VTIME] = 5;	/* 5 = 0.5-second read-timeout */
	tcsetattr(fd, TCSAFLUSH, &tty);

	for (w = 0; widths[w] != CS5; ++w)
		{
		tty.c_cflag = (tty.c_cflag & ~CSIZE) | widths[w]; 

		for (b = 0; bauds[b] != B0; ++b)
			{
			tcgetattr(fd, &tty);
			cfsetispeed(&tty, bauds[b]);
			cfsetospeed(&tty, bauds[b]);
			tcsetattr(fd, TCSAFLUSH, &tty); /* NOTE: ignore error */
			tcflush(fd, TCIOFLUSH);	/* discard unexpected data */

			for (p = magic; *p != '\033'; ++p)
				{
				ch = '\033';
				if ((write(fd, p, 1) <= 0) ||
					(tcdrain(fd) < 0) ||
					(read(fd, &ch, 1) <= 0) ||
					(ch != *p))
					{
					break;
					}
				}
			if (*p != '\033')
				break;
			}
		if (bauds[b] != B0)
			break;
		}

	/* reset port mode:
	 */
	tcflush(fd, TCIOFLUSH);

	if (widths[w] != CS5)
		return (NO);
	return (YES);
	}

INT pdl_open(const CHAR *pathname)
	{
	INT fd, e, bits;

	if ((fd = open(pathname, O_RDWR|O_NOCTTY|O_NONBLOCK, 0)) < 0)
		{
		return (-1);
		}
	if (!isatty(fd))
		{
		e = ENOTTY;
		close(fd);
		errno = e;
		return (-1);
		}
	if ((bits = fcntl(fd, F_GETFL)) >= 0)
		fcntl(fd, F_SETFL, bits & ~O_NONBLOCK);

	/* remember original tty modes: */
	if ((tcgetattr(fd, &saved_tty) != 0) ||
		(tcgetattr(fd, &tty) != 0))
		{
		e = errno;
		close(fd);
		errno = e;
		return (-1);
		}

	/* set raw-mode and other modes needed for the PedalBox:
	 */
	tty.c_iflag = (IGNBRK|IGNPAR);
	tty.c_cflag = (CREAD|HUPCL|CS8|CSTOPB|CLOCAL);
	cfsetispeed(&tty, B1200);
	cfsetospeed(&tty, B1200);
	tty.c_lflag = 0;
	if (tcsetattr(fd, TCSAFLUSH, &tty) < 0)
		{
		if (!errno)
			errno = ENOTTY;
		e = errno;
		close(fd);
		errno = e;
		return (-1);
		}

	if (!pdl_probe(fd))
		error("pedalbox is not connected on %s", pathname);
	remark("pedalbox detected on %s", pathname);

	/* write(STDERR_FILENO, "\7", 1); */
	return (fd);
	}

INT pdl_close(INT fd)
	{
	if (!isatty(fd) || (fd < 0))
		return (-1);

	/* restore original tty modes: */
	tcsetattr(fd, TCSANOW, &saved_tty);

	close(fd);
	return (-1);
	}

INT pdl_sense(INT fd)
	{
	extern int ioctl();
	INT pedals;

	#if defined(__sun) && defined(__SVR4)	/* Solaris 5.x */
	INT bits;

	if (!pdl_ping(fd))
		return (PDL_ERROR);
	if (ioctl(fd, TIOCMGET, &bits) < 0)
		{
		if (!errno)
			errno = ENOTTY;
		return (PDL_ERROR);
		}

	pedals = 0;
	if (bits & TIOCM_DSR)
		pedals |= PDL_OTHER;
	if (bits & TIOCM_CAR)
		pedals |= PDL_RIGHT;
	if (bits & TIOCM_CTS)
		pedals |= PDL_LEFT;

	#else
	#error unrecognised target platform
	#endif

	return (pedals);
	}

